﻿using Bulldozer.DrawingObjects;
using Bulldozer.Generics;
using Bulldozer.MovementStrategy;
using System.Windows.Forms;


namespace Bulldozer
{
    public partial class FormBulldozerCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly BulldozersGenericStorage _storage;


        public FormBulldozerCollection()
        {
            InitializeComponent();
            _storage = new BulldozersGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);

        }

        /// <summary>
        /// Заполнение listBoxBulldozerStorages
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxBulldozerStorages.SelectedIndex;
            listBoxBulldozerStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxBulldozerStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxBulldozerStorages.Items.Count > 0 && (index == -1 || index >= listBoxBulldozerStorages.Items.Count))
            {
                listBoxBulldozerStorages.SelectedIndex = 0;
            }
            else if (listBoxBulldozerStorages.Items.Count > 0 && index > -1 && index < listBoxBulldozerStorages.Items.Count)
            {
                listBoxBulldozerStorages.SelectedIndex = index;
            }
        }

        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddBulldozersList_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
        }

        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listBoxBulldozerStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _storage[listBoxBulldozerStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowBulldozers();
        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonDeleteBulldozersList_Click(object sender, EventArgs e)
        {
            if (listBoxBulldozerStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект {listBoxBulldozerStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxBulldozerStorages.SelectedItem.ToString() ?? string.Empty);
                ReloadObjects();
            }
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        

        private void buttonAddBulldozer_Click(object sender, EventArgs e)
        {
            if (listBoxBulldozerStorages.SelectedIndex == -1)
            {
                        MessageBox.Show("Выберите набор в списке.", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
            }
            var formBulldozerConfig = new FormBulldozerConfig();
            formBulldozerConfig.AddEvent(AddBulldozer);
            formBulldozerConfig.Show();
        }
        private void AddBulldozer(DrawingBulldozer bulldozer)
        {
            bulldozer._pictureWidth = pictureBoxCollection.Width;
            bulldozer._pictureHeight = pictureBoxCollection.Height;

            if (listBoxBulldozerStorages.SelectedIndex == -1) return;

            var obj = _storage[listBoxBulldozerStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (obj + bulldozer > -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBoxCollection.Image = obj.ShowBulldozers();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }


        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonDeleteBulldozer_Click(object sender, EventArgs e)
        {
            if (listBoxBulldozerStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxBulldozerStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(textBoxDeletingBulldozer.Text);
            if (obj - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowBulldozers();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonUpdateCollection_Click(object sender, EventArgs e)
        {
            if (listBoxBulldozerStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxBulldozerStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowBulldozers();
        }

        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.SaveData(saveFileDialog.FileName))
                {
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.LoadData(openFileDialog.FileName))
                {
                    MessageBox.Show("Загрузка прошла успешно!", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    ReloadObjects();
                }
                else
                {
                    MessageBox.Show("Не загрузилось!", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);

                }
            }
        }
    }
}
