﻿using Bulldozer.Entities;
using Bulldozer.DrawingObjects;

namespace Bulldozer
{
    public partial class FormBulldozerConfig : Form
    {
        /// <summary>
        /// Переменная-выбранный бульдозер
        /// </summary>
        DrawingBulldozer? _bulldozer = null;

        /// <summary>
        /// Делегат для передачи объекта-бульдозера
        /// </summary>
        private event Action<DrawingBulldozer>? EventAddBulldozer;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBulldozerConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += panelColor_MouseDown;
            panelPurple.MouseDown += panelColor_MouseDown;
            panelGray.MouseDown += panelColor_MouseDown;
            panelGreen.MouseDown += panelColor_MouseDown;
            panelRed.MouseDown += panelColor_MouseDown;
            panelWhite.MouseDown += panelColor_MouseDown;
            panelYellow.MouseDown += panelColor_MouseDown;
            panelBlue.MouseDown += panelColor_MouseDown;
            buttonCancel.Click += (s, e) => Close();
        }

        private void DrawBulldozer()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _bulldozer?.SetPosition(5, 5);
            _bulldozer?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawingBulldozer> ev)
        {
            if (EventAddBulldozer == null)
            {
                EventAddBulldozer = ev;
            }
            else
            {
                EventAddBulldozer += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void labelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _bulldozer = new DrawingBulldozer((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Cyan, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _bulldozer = new DrawingBulldozerUpgraded((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black, Color.Blue,checkBoxRipper.Checked, checkBoxBlade.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawBulldozer();
        }

        public void panelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void labelColor_dragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void labelColor_dragDrop(object sender, DragEventArgs e)
        {
            if (_bulldozer == null)
                return;
            switch (((Label)sender).Name)
            {
                case "labelColor":
                    _bulldozer?.EntityBulldozer?.SetBodyColor((Color) e.Data.GetData(typeof(Color)));
                    break;
                case "labelAddColor":
                    if (!(_bulldozer is DrawingBulldozerUpgraded))
                        return;
                    (_bulldozer.EntityBulldozer as EntityBulldozerUpgraded)?.SetDopColor(dopColor: (Color) e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawBulldozer();
        }
        /// <summary>
        /// Добавление бульдозера
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            EventAddBulldozer?.Invoke(_bulldozer);
            Close();
        }
    }
}
